@@ -11,6 +11,7 @@ __license__ = "GPL v2 or later"
 # standard libs
 import sys
 import logging
+import datetime as pydt
 
 
 if __name__ == '__main__':
@@ -36,6 +37,8 @@ from Gnumed.business import gmEMRStructItems
 from Gnumed.business import gmMedication
 from Gnumed.business import gmVaccination
 from Gnumed.business import gmFamilyHistory
+from Gnumed.business import gmExternalCare
+from Gnumed.business import gmOrganization
 from Gnumed.business.gmDemographicRecord import get_occupations
 
 
@@ -60,39 +63,29 @@ def set_func_ask_user(a_func = None):
 #============================================================
 class cClinicalRecord(object):
 
-	def __init__(self, aPKey=None, allow_user_interaction=True):
+	def __init__(self, aPKey=None, allow_user_interaction=True, encounter=None):
 		"""Fails if
 
 		- no connection to database possible
 		- patient referenced by aPKey does not exist
 		"""
 		self.pk_patient = aPKey			# == identity.pk == primary key
-
-		# FIXME: delegate to worker thread
-		# log access to patient record (HIPAA, for example)
-		cmd = u'SELECT gm.log_access2emr(%(todo)s)'
-		args = {'todo': u'patient [%s]' % aPKey}
-		gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
+		self.gender = None
+		self.dob = None
 
 		from Gnumed.business import gmPraxis
 		global _here
 		if _here is None:
 			_here = gmPraxis.gmCurrentPraxisBranch()
 
-		# load current or create new encounter
-		if _func_ask_user is None:
-			_log.error('[_func_ask_user] is None')
-			print "*** GNUmed [%s]: _func_ask_user is not set ***" % self.__class__.__name__
-
-		# FIXME: delegate to worker thread ?
-		self.remove_empty_encounters()
-
 		self.__encounter = None
-		if not self.__initiate_active_encounter(allow_user_interaction = allow_user_interaction):
-			raise gmExceptions.ConstructorError, "cannot activate an encounter for patient [%s]" % aPKey
-
-		# FIXME: delegate to worker thread
-		gmAllergy.ensure_has_allergy_state(encounter = self.current_encounter['pk_encounter'])
+		if encounter is None:
+			self.__old_style_init()
+		else:
+			if not isinstance(encounter, gmEMRStructItems.cEncounter):
+				raise TypeError('not an encounter: %s' % encounter)
+			#self.__encounter = encounter
+			self.active_encounter = encounter
 
 		# register backend notification interests
 		# (keep this last so we won't hang on threads when
@@ -102,13 +95,46 @@ class cClinicalRecord(object):
 
 		_log.debug('Instantiated clinical record for patient [%s].' % self.pk_patient)
 	#--------------------------------------------------------
-	def __del__(self):
-		pass
+	def __old_style_init(self):
+
+		_log.error('%s.__old_style_init() used', self.__class__.__name__)
+		print u'*** GNUmed [%s]: __old_style_init() used ***' % self.__class__.__name__
+
+		# FIXME: delegate to worker thread
+		# log access to patient record (HIPAA, for example)
+		cmd = u'SELECT gm.log_access2emr(%(todo)s)'
+		args = {'todo': u'patient [%s]' % aPKey}
+		gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
+
+		# load current or create new encounter
+		if _func_ask_user is None:
+			_log.error('[_func_ask_user] is None')
+			print "*** GNUmed [%s]: _func_ask_user is not set ***" % self.__class__.__name__
+
+#		# FIXME: delegate to worker thread ?
+		self.remove_empty_encounters()
+
+		self.__encounter = None
+		if not self.__initiate_active_encounter(allow_user_interaction = allow_user_interaction):
+			raise gmExceptions.ConstructorError, "cannot activate an encounter for patient [%s]" % aPKey
+
+#		# FIXME: delegate to worker thread
+		gmAllergy.ensure_has_allergy_state(encounter = self.current_encounter['pk_encounter'])
+
 	#--------------------------------------------------------
 	def cleanup(self):
 		_log.debug('cleaning up after clinical record for patient [%s]' % self.pk_patient)
+		if self.__encounter is not None:
+			self.__encounter.unlock(exclusive = False)
 		return True
 	#--------------------------------------------------------
+	def log_access(self, action=None):
+		if action is None:
+			action = u'EMR access for pk_identity [%s]' % self.pk_patient
+		args = {'action': action}
+		cmd = u'SELECT gm.log_access2emr(%(action)s)'
+		gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
+	#--------------------------------------------------------
 	# messaging
 	#--------------------------------------------------------
 	def _register_interests(self):
@@ -135,8 +161,11 @@ class cClinicalRecord(object):
 		# we must fail propagating the change, however, if
 		# there are local changes
 		if self.current_encounter.is_modified():
-			_log.debug('unsaved changes in active encounter, cannot switch to another one')
-			raise ValueError('unsaved changes in active encounter, cannot switch to another one')
+			_log.error('current in client: %s', self.current_encounter)
+			raise ValueError('unsaved changes in active encounter [%s], cannot switch to another one [%s]' % (
+				self.current_encounter['pk_encounter'],
+				curr_enc_in_db['pk_encounter']
+			))
 
 		if self.current_encounter.same_payload(another_object = curr_enc_in_db):
 			_log.debug('clin.encounter_mod_db received but no change to active encounter payload')
@@ -179,6 +208,87 @@ class cClinicalRecord(object):
 			relation = relation
 		)
 	#--------------------------------------------------------
+	# API: pregnancy
+	#--------------------------------------------------------
+	def _get_gender(self):
+		if self.__gender is not None:
+			return self.__gender
+		cmd = u'SELECT gender, dob FROM dem.v_basic_person WHERE pk_identity = %(pat)s'
+		args = {u'pat': self.pk_patient}
+		rows, idx = gmPG2.run_ro_queries(queries = [{u'cmd': cmd, u'args': args}], get_col_idx = False)
+		self.__gender = rows[0][u'gender']
+		self.__dob = rows[0][u'dob']
+
+	def _set_gender(self, gender):
+		self.__gender = gender
+
+	gender = property(_get_gender, _set_gender)
+	#--------------------------------------------------------
+	def _get_dob(self):
+		if self.__dob is not None:
+			return self.__dob
+		cmd = u'SELECT gender, dob FROM dem.v_basic_person WHERE pk_identity = %(pat)s'
+		args = {u'pat': self.pk_patient}
+		rows, idx = gmPG2.run_ro_queries(queries = [{u'cmd': cmd, u'args': args}], get_col_idx = False)
+		self.__gender = rows[0][u'gender']
+		self.__dob = rows[0][u'dob']
+
+	def _set_dob(self, dob):
+		self.__dob = dob
+
+	dob = property(_get_dob, _set_dob)
+	#--------------------------------------------------------
+	def _get_EDC(self):
+		cmd = u'SELECT edc FROM clin.patient WHERE fk_identity = %(pat)s'
+		args = {'pat': self.pk_patient}
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = False)
+		if len(rows) == 0:
+			return None
+		return rows[0]['edc']
+
+	def _set_EDC(self, edc):
+		cmd = u"""
+			INSERT INTO clin.patient (fk_identity, edc) SELECT
+				%(pat)s,
+				%(edc)s
+			WHERE NOT EXISTS (
+				SELECT 1 FROM clin.patient WHERE fk_identity = %(pat)s
+			)
+			RETURNING pk"""
+		args = {'pat': self.pk_patient, 'edc': edc}
+		rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = False, return_data = True)
+		if len(rows) == 0:
+			cmd = u'UPDATE clin.patient SET edc = %(edc)s WHERE fk_identity = %(pat)s'
+			gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
+
+	EDC = property(_get_EDC, _set_EDC)
+	#--------------------------------------------------------
+	def _EDC_is_fishy(self):
+		edc = self.EDC
+		if edc is None:
+			return False
+		if self.gender != u'f':
+			return True
+		now = gmDateTime.pydt_now_here()
+		# mother too young
+		if (self.dob + pydt.timedelta(weeks = 5 * 52)) > now:
+			return True
+		# mother too old
+		if (self.dob + pydt.timedelta(weeks = 55 * 52)) < now:
+			return True
+		# Beulah Hunter, 375 days (http://www.reference.com/motif/health/longest-human-pregnancy-on-record)
+		# EDC too far in the future
+		if (edc - pydt.timedelta(days = 380)) > now:
+			return True
+		# even if the pregnancy would have *started* when it
+		# was documented to *end* it would be over by now by
+		# all accounts
+		# EDC too far in the past
+		if edc < (now - pydt.timedelta(days = 380)):
+			return True
+
+	EDC_is_fishy = property(_EDC_is_fishy, lambda x:x)
+	#--------------------------------------------------------
 	# API: performed procedures
 	#--------------------------------------------------------
 	def get_performed_procedures(self, episodes=None, issues=None):
@@ -207,6 +317,14 @@ class cClinicalRecord(object):
 			procedure = procedure
 		)
 	#--------------------------------------------------------
+	def get_procedure_locations_as_org_units(self):
+		where = u'pk_org_unit IN (SELECT DISTINCT pk_org_unit FROM clin.v_procedures_not_at_hospital WHERE pk_patient = %(pat)s)'
+		args = {'pat': self.pk_patient}
+		cmd = gmOrganization._SQL_get_org_unit % where
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
+		return [ gmOrganization.cOrgUnit(row = {'pk_field': 'pk_org_unit', 'data': r, 'idx': idx}) for r in rows ]
+
+	#--------------------------------------------------------
 	# API: hospitalizations
 	#--------------------------------------------------------
 	def get_hospital_stays(self, episodes=None, issues=None, ongoing_only=False):
@@ -247,6 +365,14 @@ class cClinicalRecord(object):
 		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
 		return rows
 	#--------------------------------------------------------
+	def get_attended_hospitals_as_org_units(self):
+		where = u'pk_org_unit IN (SELECT DISTINCT pk_org_unit FROM clin.v_hospital_stays WHERE pk_patient = %(pat)s)'
+		args = {'pat': self.pk_patient}
+		cmd = gmOrganization._SQL_get_org_unit % where
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
+		return [ gmOrganization.cOrgUnit(row = {'pk_field': 'pk_org_unit', 'data': r, 'idx': idx}) for r in rows ]
+
+	#--------------------------------------------------------
 	# API: narrative
 	#--------------------------------------------------------
 	def add_notes(self, notes=None, episode=None, encounter=None):
@@ -266,13 +392,14 @@ class cClinicalRecord(object):
 
 		return True
 	#--------------------------------------------------------
-	def add_clin_narrative(self, note='', soap_cat='s', episode=None):
+	def add_clin_narrative(self, note='', soap_cat='s', episode=None, link_obj=None):
 		if note.strip() == '':
 			_log.info('will not create empty clinical note')
 			return None
 		if isinstance(episode, gmEMRStructItems.cEpisode):
 			episode = episode['pk_episode']
 		status, data = gmClinNarrative.create_clin_narrative (
+			link_obj = link_obj,
 			narrative = note,
 			soap_cat = soap_cat,
 			episode_id = episode,
@@ -283,13 +410,9 @@ class cClinicalRecord(object):
 			return None
 		return data
 	#--------------------------------------------------------
-	def get_clin_narrative(self, since=None, until=None, encounters=None, episodes=None, issues=None, soap_cats=None, providers=None):
+	def get_clin_narrative(self, encounters=None, episodes=None, issues=None, soap_cats=None, providers=None):
 		"""Get SOAP notes pertinent to this encounter.
 
-			since
-				- initial date for narrative items
-			until
-				- final date for narrative items
 			encounters
 				- list of encounters whose narrative are to be retrieved
 			episodes
@@ -328,7 +451,7 @@ class cClinicalRecord(object):
 
 		if encounters is not None:
 			where_parts.append(u'pk_encounter IN %(encs)s')
-			if len(episodes) == 0:
+			if len(encounters) == 0:
 				args['encs'] = tuple()
 			else:
 				if isinstance(encounters[0], gmEMRStructItems.cEncounter):
@@ -361,12 +484,6 @@ class cClinicalRecord(object):
 
 		filtered_narrative = [ gmClinNarrative.cNarrative(row = {'pk_field': 'pk_narrative', 'idx': idx, 'data': row}) for row in rows ]
 
-		if since is not None:
-			filtered_narrative = filter(lambda narr: narr['date'] >= since, filtered_narrative)
-
-		if until is not None:
-			filtered_narrative = filter(lambda narr: narr['date'] < until, filtered_narrative)
-
 		if providers is not None:
 			filtered_narrative = filter(lambda narr: narr['modified_by'] in providers, filtered_narrative)
 
@@ -648,7 +765,7 @@ order by
 	#--------------------------------------------------------
 	def format_summary(self):
 
-		cmd = u"SELECT dob from dem.v_basic_person where pk_identity = %(pk)s"
+		cmd = u"SELECT dob FROM dem.v_basic_person WHERE pk_identity = %(pk)s"
 		args = {'pk': self.pk_patient}
 		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = False)
 		dob = rows[0]['dob']
@@ -762,6 +879,21 @@ order by
 				gmTools.u_right_double_angle_quote
 			)
 
+		care = self.get_external_care_items(order_by = u'issue, organization, unit, provider')
+		if len(care) > 0:
+			txt += u'\n'
+			txt += _('External care')
+			txt += u'\n'
+		for item in care:
+			txt += u' %s: %s\n' % (
+				item['issue'],
+				gmTools.coalesce (
+					item['provider'],
+					u'%s@%s' % (item['unit'], item['organization']),
+					u'%%s (%s@%s)' % (item['unit'], item['organization'])
+				)
+			)
+
 		return txt
 	#--------------------------------------------------------
 	def format_as_journal(self, left_margin=0, patient=None):
@@ -926,6 +1058,11 @@ WHERE
 
 	allergy_state = property(_get_allergy_state, _set_allergy_state)
 	#--------------------------------------------------------
+	# API: external care
+	#--------------------------------------------------------
+	def get_external_care_items(self, order_by=None):
+		return gmExternalCare.get_external_care_items(pk_identity = self.pk_patient, order_by = order_by)
+	#--------------------------------------------------------
 	# API: episodes
 	#--------------------------------------------------------
 	def get_episodes(self, id_list=None, issues=None, open_status=None, order_by=None, unlinked_only=False):
@@ -994,12 +1131,13 @@ WHERE
 			epis.append(row[0])
 		return self.get_episodes(id_list=epis)
 	#------------------------------------------------------------------
-	def add_episode(self, episode_name=None, pk_health_issue=None, is_open=False, allow_dupes=False):
+	def add_episode(self, episode_name=None, pk_health_issue=None, is_open=False, allow_dupes=False, link_obj=None):
 		"""Add episode 'episode_name' for a patient's health issue.
 
 		- silently returns if episode already exists
 		"""
 		episode = gmEMRStructItems.create_episode (
+			link_obj = link_obj,
 			pk_health_issue = pk_health_issue,
 			episode_name = episode_name,
 			is_open = is_open,
@@ -1560,8 +1698,11 @@ WHERE
 			_log.debug('switching of active encounter')
 			# fail if the currently active encounter has unsaved changes
 			if self.__encounter.is_modified():
-				_log.debug('unsaved changes in active encounter, cannot switch to another one')
-				raise ValueError('unsaved changes in active encounter, cannot switch to another one')
+				_log.error('current in client: %s', self.__encounter)
+				raise ValueError('unsaved changes in active encounter [%s], cannot switch to another one [%s]' % (
+					self.__encounter['pk_encounter'],
+					encounter['pk_encounter']
+				))
 
 		# be more conservative, it seems to have brought about
 		# races involving encounter mod signals which made GNUmed crash
@@ -1993,6 +2134,7 @@ SELECT MIN(earliest) FROM (
 		return gmEMRStructItems.cEncounter(row = {'data': rows[0], 'idx': idx, 'pk_field': 'pk_encounter'})
 	#------------------------------------------------------------------
 	def remove_empty_encounters(self):
+		_log.debug('removing empty encounters for pk_identity [%s]', self.pk_patient)
 		cfg_db = gmCfg.cCfgSQL()
 		ttl = cfg_db.get2 (
 			option = u'encounter.ttl_if_empty',
@@ -2000,7 +2142,6 @@ SELECT MIN(earliest) FROM (
 			bias = u'user',
 			default = u'1 week'
 		)
-
 #		# FIXME: this should be done async
 		cmd = u"SELECT clin.remove_old_empty_encounters(%(pat)s::INTEGER, %(ttl)s::INTERVAL)"
 		args = {'pat': self.pk_patient, 'ttl': ttl}
@@ -2008,8 +2149,10 @@ SELECT MIN(earliest) FROM (
 			rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
 		except:
 			_log.exception('error deleting empty encounters')
+			return False
 
 		return True
+
 	#------------------------------------------------------------------
 	# API: measurements / test results
 	#------------------------------------------------------------------
@@ -2030,6 +2173,13 @@ SELECT MIN(earliest) FROM (
 			patient = self.pk_patient
 		)
 	#------------------------------------------------------------------
+	def get_results_for_day(self, timestamp=None, order_by=None):
+		return gmPathLab.get_results_for_day (
+			timestamp = timestamp,
+			patient = self.pk_patient,
+			order_by = order_by
+		)
+	#------------------------------------------------------------------
 	def get_unsigned_results(self, order_by=None):
 		if order_by is None:
 			order_by = u''
@@ -2148,7 +2298,7 @@ SELECT MIN(earliest) FROM (
 
 		return tests
 	#------------------------------------------------------------------
-	def add_test_result(self, episode=None, type=None, intended_reviewer=None, val_num=None, val_alpha=None, unit=None):
+	def add_test_result(self, episode=None, type=None, intended_reviewer=None, val_num=None, val_alpha=None, unit=None, link_obj=None):
 
 		try:
 			epi = int(episode)
@@ -2160,10 +2310,8 @@ SELECT MIN(earliest) FROM (
 		except:
 			type = type['pk_test_type']
 
-		if intended_reviewer is None:
-			intended_reviewer = gmStaff.gmCurrentProvider()['pk_staff']
-
 		tr = gmPathLab.create_test_result (
+			link_obj = link_obj,
 			encounter = self.current_encounter['pk_encounter'],
 			episode = epi,
 			type = type,
@@ -2175,6 +2323,16 @@ SELECT MIN(earliest) FROM (
 
 		return tr
 	#------------------------------------------------------------------
+	def get_labs_as_org_units(self):
+		where = u'pk_org_unit IN (%s)' % u"""
+			SELECT DISTINCT fk_org_unit FROM clin.test_org WHERE pk IN (
+				SELECT DISTINCT pk_test_org FROM clin.v_test_results where pk_patient = %(pat)s
+			)"""
+		args = {'pat': self.pk_patient}
+		cmd = gmOrganization._SQL_get_org_unit % where
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
+		return [ gmOrganization.cOrgUnit(row = {'pk_field': 'pk_org_unit', 'data': r, 'idx': idx}) for r in rows ]
+
 	#------------------------------------------------------------------
 	#------------------------------------------------------------------
 	#------------------------------------------------------------------
