@@ -50,7 +50,7 @@ care of all the pre- and post-GUI runtime environment setup.
  Show this help.
 """
 #==========================================================
-__author__  = "H. Herb <hherb@gnumed.net>, K. Hilbert <Karsten.Hilbert@gmx.net>, I. Haywood <i.haywood@ugrad.unimelb.edu.au>"
+__author__ = "H. Herb <hherb@gnumed.net>, K. Hilbert <Karsten.Hilbert@gmx.net>, I. Haywood <i.haywood@ugrad.unimelb.edu.au>"
 __license__ = "GPL v2 or later (details at http://www.gnu.org)"
 
 # standard library
@@ -86,8 +86,8 @@ against. Please run GNUmed as a non-root user.
 	sys.exit(1)
 
 #----------------------------------------------------------
-current_client_version = u'1.4.15'
-current_client_branch = u'1.4'
+current_client_version = u'1.5.5'
+current_client_branch = u'1.5'
 
 _log = None
 _cfg = None
@@ -245,6 +245,19 @@ def setup_local_repo_path():
 	sys.path.insert(0, local_repo_path)
 	_log.debug('sys.path with repo:')
 	_log.debug(sys.path)
+
+#==========================================================
+def setup_fault_handler(target=None):
+	try:
+		import faulthandler
+	except ImportError:
+		print "Faulthandler not available ..."
+		return
+	if target is None:
+		faulthandler.enable()
+		return
+	faulthandler.enable(file = target)
+
 #==========================================================
 def setup_logging():
 	try:
@@ -252,6 +265,8 @@ def setup_logging():
 	except ImportError:
 		sys.exit(import_error_sermon % '\n '.join(sys.path))
 
+	setup_fault_handler(target = _gmLog2._logfile)
+
 	global gmLog2
 	gmLog2 = _gmLog2
 
@@ -259,7 +274,6 @@ def setup_logging():
 	_log = logging.getLogger('gm.launcher')
 #==========================================================
 def log_startup_info():
-	_log.info(u'Starting up as main module.')
 	_log.info(u'GNUmed client version [%s] on branch [%s]', current_client_version, current_client_branch)
 	_log.info(u'Platform: %s', platform.uname())
 	_log.info(u'Python %s on %s (%s)', sys.version, sys.platform, os.name)
@@ -597,6 +611,7 @@ def shutdown_tmp_dir():
 #==========================================================
 # main - launch the GNUmed wxPython GUI client
 #----------------------------------------------------------
+setup_fault_handler(target = None)
 setup_python_path()
 setup_logging()
 log_startup_info()
