@@ -111,6 +111,18 @@ except ImportError:
 	print "  INFO : GNUmed will work but you will be unable"
 	print "  INFO : to visualize search results and lab data"
 
+#print " simplejson...",
+#try:
+#	import simplejson
+#	print "found"
+#except ImportError:
+#	missing = True
+#	print ""
+#	print "  ERROR: simplejson not installed"
+#	print "  INFO : this is needed for accessing eGK/KVK/PKVK cards"
+#	print "  INFO : GNUmed will work but you will be unable"
+#	print "  INFO : to read German chipcards"
+
 print " libxml2...",
 try:
 	import libxml2
@@ -188,6 +200,28 @@ except ImportError:
 	print "  INFO : note that you need v0.2.1 (not 0.2.2)"
 	print "  INFO :  <pip install pysvg==0.2.1>"
 
+print " faulthandler...",
+try:
+	import faulthandler
+	print "found"
+except ImportError:
+	missing = True
+	print ""
+	print "  ERROR: faulthandler not installed"
+	print "  INFO : this is used to capture catastrophic faults"
+	print "  INFO : GNUmed will still work without it"
+
+print " vobject...",
+try:
+	import vobject
+	print "found"
+except ImportError:
+	missing = True
+	print ""
+	print "  ERROR: vobject not installed"
+	print "  INFO : this is used to handle vCard data"
+	print "  INFO : GNUmed will still work without it"
+
 if missing:
 	print ""
 	print "sys.path is currently set as follows:"
